require 'spec_helper'

RSpec.describe KarrierWave::File do
  let(:config_name) { 'config_name' }
  let(:klass) { Class.new(ActiveRecord::Base) { self.table_name = 'messages' } }
  let(:instance) { klass.new }
  let(:mounter) { KarrierWave::Mount::Mounter.new(instance, :attachment) }
  let(:client) { file.send(:client) }
  let(:mount_options) { { config_name: config_name } }

  before { klass.mount_kage :attachment, mount_options }

  describe '#store' do
    let(:file) { KarrierWave::File.new(mounter, config_name) }
    let(:access_key) { 'accesskey' }
    let(:response) { { 'access_key' => access_key } }

    context 'when it is local file' do
      let(:new_file) { KarrierWave::LocalFile.new(File.open('spec/support/1x1.jpg')) }

      it { expect(client).to receive(:store).with(new_file.path, 'image/jpeg') { response }; file.store(new_file) }

      context 'after successful store' do
        before { allow(client).to receive(:store).with(new_file.path, 'image/jpeg') { response } }

        it { expect { file.store(new_file) }.to change { file.identifier }.to(access_key) }
        it { expect(new_file.file).to receive(:close); file.store(new_file) }
      end
    end

    context 'when it is remote file' do
      let(:url) { 'https://www.google.com/images/srpr/logo11w.png' }
      let(:path) { '/path/to/tempfile' }
      let(:base_uri) { URI.parse(url) }
      let(:mock_tempfile) { double(path: path, meta: {}, base_uri: base_uri, close: nil) }
      let(:new_file) { KarrierWave::RemoteFile.new(mock_tempfile) }

      before { allow_any_instance_of(URI::HTTPS).to receive(:open) { mock_tempfile } }

      it { expect(client).to receive(:store).with(new_file.path, 'image/png') { response }; file.store(new_file) }

      context 'after successful store' do
        before { allow(client).to receive(:store).with(new_file.path, 'image/png') { response } }

        it { expect { file.store(new_file) }.to change { file.identifier }.to(access_key) }
        it { expect(new_file.file).to receive(:close); file.store(new_file) }
      end
    end

    context 'when it is remote object' do
      let(:identifier) { 'a/b/c' }
      let(:file) { KarrierWave::File.new(mounter, config_name) }
      let(:new_file) { KarrierWave::RemoteObject.new(identifier) }

      it { expect(client).to receive(:copy).with(identifier) { response }; file.store(new_file) }

      context 'after successful store' do
        before { allow(client).to receive(:copy).with(identifier) { response } }

        it { expect { file.store(new_file) }.to change { file.identifier }.to(access_key) }
      end
    end
  end

  describe '#info' do
    let(:identifier) { 'accesskey' }
    let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier) }

    it { expect(client).to receive(:info).with(identifier); file.info }
  end

  describe '#delete' do
    let(:identifier) { 'accesskey' }
    let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier) }

    it { expect(client).to receive(:remove).with(identifier); file.delete }
    it { expect(client.admin_connection).to receive(:post) { double(success?: true) }; file.delete }

    context 'when deletion API request was succesful' do
      before { allow(client.admin_connection).to receive(:post) { double(success?: true) } }

      it { expect { file.delete }.to change { file.identifier }.to(nil) }
      it { expect { file.delete }.to change { file.to_json }.to(nil) }
    end
  end

  describe '#meta' do
    let(:identifier) { 'accesskey' }
    let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier) }

    it { expect(client).to receive(:meta).with(identifier); file.meta }
  end

  describe '#type' do
    let(:identifier) { 'accesskey' }
    let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier) }
    let(:new_file) { KarrierWave::LocalFile.new(File.open('spec/support/1x1.jpg')) }

    it { expect(file.type).to eq(:image) }

    context 'after successful store' do
      before { file.store(new_file) }

      context 'when no type is set' do
        it { expect(file.type).to eq(:image) }
      end

      context 'when it is animated gif' do
        before do
          allow_any_instance_of(KarrierWave::Client).to receive(:meta) do
            { "original" => { "filename" => "img.gif", "length" => 1620409, "content_type"=> "image/gif", "width" => 500, "height"=> 375, "ani"=>1 },
              "xlarge" => { "filename" => "img_xl.jpg", "length" => 30875, "content_type"=> "image/jpeg", "width" => 500, "height"=> 375 } }
          end
        end

        context 'when disallow_animated_gif is not set' do
          before { KarrierWave.config.disallow_animated_gif = nil }

          it { expect(file.type).to eq(:animated) }
        end

        context 'when disallow_animated_gif is not set' do
          before { KarrierWave.config.disallow_animated_gif = true }

          it { expect(file.type).to eq(:image) }
        end
      end

      context 'when type is specified as video' do
        let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier, type: :video) }

        it { expect(file.type).to eq(:video) }
      end
    end
  end

  describe '#dimension' do
    let(:identifier) { 'accesskey' }

    context 'after successful store' do
      let(:new_file) { KarrierWave::LocalFile.new(File.open('spec/support/1x1.jpg')) }

      before { file.store(new_file) }

      context 'when type is image' do
        let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier) }

        context 'when ignore_dimensions is set' do
          before { KarrierWave.config.ignore_dimensions = true }

          it { expect(file.dimension[:width]).to be_nil }
          it { expect(file.dimension[:height]).to be_nil }
        end

        context 'when ignore_dimensions is not set' do
          before { KarrierWave.config.ignore_dimensions = nil }

          it { expect(file.dimension[:width]).to eq(442) }
          it { expect(file.dimension[:height]).to eq(247) }
        end
      end

      context 'when type is video' do
        let(:file) { KarrierWave::File.new(mounter, config_name, identifier: identifier, type: :video) }

        before do
          allow_any_instance_of(KarrierWave::Client).to receive(:meta) do
            { "video"=> { "filename"=>"klogv_720.mp4", "resolution"=>"1280x720" },
              "preview"=> [
                {"filename"=>"image0.jpg", "length"=>126997, "content_type"=>"image/jpeg", "width"=>1280, "height"=>720, "offset"=>"0"},
                {"filename"=>"image1.jpg", "length"=>99928, "content_type"=>"image/jpeg", "width"=>1280, "height"=>720, "offset"=>"5"},
                {"filename"=>"image2.jpg", "length"=>105852, "content_type"=>"image/jpeg", "width"=>1280, "height"=>720, "offset"=>"10"} ]}
          end
        end

        context 'when ignore_dimensions is set' do
          before { KarrierWave.config.ignore_dimensions = true }

          it { expect(file.dimension[:width]).to be_nil }
          it { expect(file.dimension[:height]).to be_nil }
        end

        context 'when ignore_dimensions is not set' do
          before { KarrierWave.config.ignore_dimensions = nil }

          it { expect(file.dimension[:width]).to eq(1280) }
          it { expect(file.dimension[:height]).to eq(720) }
        end
      end
    end
  end
end
